/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2010 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceDoesNotExistException;
import org.ow2.petals.jmx.exception.RouterMonitorServiceErrorException;

/**
 * @author rnaudin
 * 
 */
public class RouterMonitorServiceClient extends AbstractServiceClient {

    /**
     * JMX name
     */
    private final static String ROUTER_MONITOR_JMX_NAME = "RouterMonitor";

    /**
     * Action : getExchangeHistory
     */
    private static final String GET_EXCHANGE_HISTORY = "getExchangeHistory";

    /**
     * Action : getExchange
     */
    private static final String GET_EXCHANGE = "getExchange";

    /**
     * Action : getExchangeIds
     */
    private static final String GET_EXCHANGE_IDS = "getExchangeIds";

    /**
     * Action : getExchanges
     */
    private static final String GET_EXCHANGES = "getExchanges";

    /**
     * Action : monitorExchanges
     */
    private static final String MONITOR_EXCHANGES = "monitorExchanges";

    /**
     * Action : unmonitorExchanges
     */
    private static final String UNMONITOR_EXCHANGES = "unmonitorExchanges";

    /**
     * Action : getmonitoredExchanges
     */
    private static final String GET_MONITORED_EXCHANGES = "getMonitoredExchanges";

    /**
     * Creates a new instance of {@link RouterMonitorServiceClient}
     * 
     * @param petalsDomain
     * @param mBeanServerConnection
     */
    public RouterMonitorServiceClient(String petalsDomain,
            MBeanServerConnection mBeanServerConnection)
            throws RouterMonitorServiceDoesNotExistException, RouterMonitorServiceErrorException {
        super(petalsDomain, mBeanServerConnection);

        try {
            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, ROUTER_MONITOR_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new RouterMonitorServiceDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (IOException e) {
            throw new RouterMonitorServiceErrorException(e);
        }
    }

    /**
     * 
     * Get the exchange with the given exchange Id.
     * 
     * @param exchangeId
     * @return The exchange in a key/value form
     * 
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public Map<String, String> getExchange(final String exchangeId)
            throws RouterMonitorServiceErrorException {
        Map<String, String> result;

        Object[] objects = new Object[] { exchangeId };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Map<String, String>) this.performAction(GET_EXCHANGE, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the history of the given exchange.
     * 
     * @param exchangeId
     *            the send/received history
     * @return The history exchange in a key/value form
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public Map<String, Long> getExchangeHistory(final String exchangeId)
            throws RouterMonitorServiceErrorException {

        Map<String, Long> result;

        Object[] objects = new Object[] { exchangeId };
        String[] strings = new String[] { String.class.getName() };

        try {
            result = (Map<String, Long>) this.performAction(GET_EXCHANGE_HISTORY, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the list of exchange Ids that has been sent or received at least once
     * and matching the duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<String> getExchangeIds(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException {

        List<String> result;

        Object[] objects = new Object[] { interfaceName, serviceName, endpointName, operationName,
                startDate, endDate };

        String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), long.class.getName(),
                long.class.getName() };

        try {
            result = (List<String>) this.performAction(GET_EXCHANGE_IDS, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the number of exchange that has been sent or received at least once
     * and matching the duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    public int getExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException {

        Integer result;

        Object[] objects = new Object[] { interfaceName, serviceName, endpointName, operationName,
                startDate, endDate };

        String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), long.class.getName(),
                long.class.getName() };

        try {
            result = (Integer) this.performAction(GET_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result.intValue();
    }

    /**
     * Get the list of Router Monitor filter.
     * 
     * @return the list of Router Monitor filter in key/values form.
     * 
     * @throws RouterMonitorServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, String>> getMonitoredExchange()
            throws RouterMonitorServiceErrorException {
        List<Map<String, String>> result;

        try {
            result = (List<Map<String, String>>) this.performAction(GET_MONITORED_EXCHANGES, null,
                    null);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Activate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param storeExchanges
     *            if {@code true}, the exchanges are stored in the Persistence
     *            Service.
     * @throws RouterMonitorServiceErrorException
     */
    public void monitorExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final boolean storeExchanges)
            throws RouterMonitorServiceErrorException {

        Object[] objects = new Object[] { interfaceName, serviceName, endpointName, operationName,
                storeExchanges };

        String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName(), boolean.class.getName() };

        try {
            this.performAction(MONITOR_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }
    }

    /**
     * Deactivate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param ServiceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public void unmonitorExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName) throws RouterMonitorServiceErrorException {

        Object[] objects = new Object[] { interfaceName, serviceName, endpointName, operationName };

        String[] strings = new String[] { QName.class.getName(), QName.class.getName(),
                String.class.getName(), QName.class.getName() };

        try {
            this.performAction(UNMONITOR_EXCHANGES, objects, strings);
        } catch (final PerformActionErrorException e) {
            throw new RouterMonitorServiceErrorException(e);
        } catch (final ClassCastException e) {
            throw new RouterMonitorServiceErrorException("Unexpected result type");
        }
    }
}
